import os
import glob
import re
import time
import ftfy
from bs4 import BeautifulSoup
import concurrent.futures

MAX_THREADS = 8

def parse_file(filename):
    print(f"Parsing {filename}...")

    with open(filename, 'r', encoding='utf-8') as f:
        content = f.read()
        soup = BeautifulSoup(content, "html.parser")
        print(f"Parsed {filename}...")
        cleaned_posts = []

        op_post = soup.find('article', attrs={'data-thread-num' : True})
        op_post_title = op_post.find('h2', class_='post_title').text
        op_post_id = f"No.{op_post.find(title='Reply to this post').text}"
        op_post_filename = f"File: {op_post.find('a', class_='post_file_filename').text}"
        op_post_text_node = op_post.find('div', class_='text')
        for br in op_post_text_node('br'):
            br.replace_with('\n')
        op_post_text = clean_text(op_post_text_node.get_text())
        op_post_final = f"{op_post_title} - Anonymous - {op_post_id}\n{op_post_filename}\n{op_post_text}"
        cleaned_posts.append(op_post_final)
        # print(op_post_final)

        posts = soup.find_all('div', class_='post_wrapper')

        for p in posts:
            post_author = "Anonymous"
            post_id = f"No.{p.find(title='Reply to this post').text}"
            post_filename_node = p.find('a', class_='post_file_filename')
            post_filename = f"File: {post_filename_node['title']}\n" if post_filename_node is not None else ""
            post_text_node = p.find('div', class_='text')
            for br in post_text_node('br'):
                br.replace_with('\n')
            post_text = clean_text(post_text_node.get_text())
            post_final = f"{post_author} - {post_id}\n{post_filename}{post_text}"
            # if post_id == "No.277023641": print(post_final)
            cleaned_posts.append(post_final)
    all_posts = "\n".join(cleaned_posts)
    with open(f".\\cleaned\\Thread_{op_post_id}.txt", "wb") as fh:
        fh.write(all_posts.encode(encoding='UTF-8'))

def clean_text(text):
    text = text.strip()
    text = ftfy.fix_text(text).replace('…', '...').replace('\N{SOFT HYPHEN}', '').replace('\u200b', '').replace(u"\uFFFD", '').replace(u"\u009D", '').replace(u"\u0081", '') # cleanup unicode special
    text = re.sub(r'\n+', '\n', text, flags=re.M) # no extra linebreaks
    return text


html_files = glob.glob(r"./*.html")
threads = min(MAX_THREADS, len(html_files))
with concurrent.futures.ThreadPoolExecutor(max_workers=threads) as executor:
    executor.map(parse_file, html_files)
# parse_file(html_files[0])